@@ -21,8 +21,11 @@ use Net::Amazon::IAM::Users;
 use Net::Amazon::IAM::Policy;
 use Net::Amazon::IAM::Policies;
 use Net::Amazon::IAM::UserPolicy;
+use Net::Amazon::IAM::PolicyVersion;
+use Net::Amazon::IAM::PolicyVersions;
 use Net::Amazon::IAM::Group;
 use Net::Amazon::IAM::Groups;
+use Net::Amazon::IAM::GroupPolicy;;
 use Net::Amazon::IAM::GetGroupResult;
 use Net::Amazon::IAM::AccessKey;
 use Net::Amazon::IAM::AccessKeyMetadata;
@@ -37,7 +40,7 @@ use Net::Amazon::IAM::InstanceProfile;
 use Net::Amazon::IAM::InstanceProfiles;
 use Net::Amazon::IAM::LoginProfile;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 NAME
 
@@ -45,7 +48,7 @@ Net::Amazon::IAM - Perl interface to the Amazon Identity and Access Management.
 
 =head1 VERSION
 
-This is Net::Amazon::IAM version 0.04
+This is Net::Amazon::IAM version 0.05
 
 IAM Query API version: '2010-05-08'
 
@@ -55,23 +58,10 @@ IAM Query API version: '2010-05-08'
 
  my $iam = Net::Amazon::IAM->new(
    AWSAccessKeyId  => 'PUBLIC_KEY_HERE',
-   SecretAccessKey => 'SECRET_KEY_HERE'
+   SecretAccessKey => 'SECRET_KEY_HERE',
+   return_errors   => 0, # which is default
  );
 
- # create new user
- my $user = $iam->create_user (
-   UserName => 'testuser',
-   Path     => '/path/to/test/user/',
- );
-
- # delete user
- my $delete = $iam->delete_user(UserName => 'testuser');
- if($delete->isa("Net::Amazon::IAM::Error")) {
-   print Dumper $delete;
- }else{
-   print "User was successfuly deleted\n";
- }
-
  # prepare user policy document
  my $policy_document = {
    Version => '2012-10-17',
@@ -90,20 +80,27 @@ IAM Query API version: '2010-05-08'
    ],
  };
 
- # attach the document to the user
- my $policy = $iam->put_user_policy (
-    PolicyName     => 'somtestpolicy',
-    UserName       => 'sometestuser',
-    PolicyDocument => $policy_document,
- );
+ try {
+   # create new user
+   my $user = $iam->create_user(
+      UserName => 'testuser',
+      Path     => '/path/to/test/users/',
+   );
+   
+   # Add an inline user policy document.
+   my $policy = $iam->put_user_policy (
+      PolicyName     => 'somtestpolicy',
+      UserName       => 'sometestuser',
+      PolicyDocument => $policy_document,
+   );
 
- if($policy->isa("Net::Amazon::IAM::Error")) {
-   print Dumper $policy;
- }else{
-   print "Policy was added\n";
+   print $user->UserId . "\n";
+   print $policy->PolicyId . "\n";
+ } catch {
+   my $error = shift();
+   print $error->as_string() . "\n";
  }
 
-
 If an error occurs while communicating with IAM, these methods will
 throw a L<Net::Amazon::IAM::Error> exception.
 
@@ -406,7 +403,7 @@ Where to create new user
 
 =back
 
-Returns a Net::Amazon::IAM::User object on success or Net::Amazon::IAM::Error on fail.
+Returns a L<Net::Amazon::IAM::User> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -441,7 +438,7 @@ What user should be deleted
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -473,7 +470,7 @@ New user username
 
 =back
 
-Returns a Net::Amazon::IAM::User object on success or Net::Amazon::IAM::Error on fail.
+Returns a L<Net::Amazon::IAM::User> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -515,7 +512,7 @@ New name for the user. Include this parameter only if you're changing the user's
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -565,7 +562,7 @@ names whose path starts with /division_abc/subdivision_xyz/.
 
 =back
 
-Returns Net::Amazon::IAM::Users object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Users> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -610,7 +607,7 @@ The name of the user to add.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -647,7 +644,7 @@ The name of the user to remove.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -684,7 +681,7 @@ The path to the group.
 
 =back
 
-Returns Net::Amazon::IAM::Group object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Group> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -732,7 +729,7 @@ Set it to the value of the Marker element in the response you just received.
 
 =back
 
-Returns Net::Amazon::IAM::GetGroupResult object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::GetGroupResult> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -778,7 +775,7 @@ The name of the group to delete.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -824,7 +821,7 @@ gets all groups whose path starts with /division_abc/subdivision_xyz/.
 
 =back
 
-Returns Net::Amazon::IAM::Groups object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Groups> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -877,7 +874,7 @@ The path for the policy.
 
 =back
 
-Returns Net::Amazon::IAM::Policy object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Policy> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -916,7 +913,7 @@ The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
 
 =back
 
-Returns Net::Amazon::IAM::Policy object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Policy> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -950,7 +947,7 @@ The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1030,7 +1027,7 @@ Example:
 
 =back
 
-Returns Net::Amazon::IAM::Policies on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Policies> on success or L<Net::Amazon::IAM::Error> on fail.
 When no policies found, the Policies attribute will be just empty array.
 
 =cut
@@ -1062,9 +1059,231 @@ sub list_policies {
    }
 }
 
+=head2 get_policy_version(%params)
+
+Retrieves information about the specified version of the specified 
+managed policy, including the policy document.
+
+=over
+
+=item PolicyArn (required)
+
+The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
+
+=item VersionId (required)
+
+Identifies the policy version to retrieve.
+
+=back
+
+Returns L<Net::Amazon::IAM::PolicyVersion> on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub get_policy_version {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      PolicyArn => { type => SCALAR },
+      VersionId => { type => SCALAR },
+   });
+
+   my $xml = $self->_sign(Action => 'GetPolicyVersion', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      my %result = %{$xml->{'GetPolicyVersionResult'}{'PolicyVersion'}};
+      $result{'Document'} = decode_json(URI::Encode->new()->decode($result{'Document'}));
+      return Net::Amazon::IAM::PolicyVersion->new(
+         %result,
+      );
+   }
+}
+
+=head2 set_default_policy_version(%params)
+
+Sets the specified version of the specified policy as the policy's default (operative) version.
+
+=over
+
+=item PolicyArn (required)
+
+The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
+
+=item VersionId (required)
+
+The version of the policy to set as the default (operative) version.
+
+=back
+
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub set_default_policy_version {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      PolicyArn => { type => SCALAR },
+      VersionId => { type => SCALAR },
+   });
+
+   my $xml = $self->_sign(Action => 'SetDefaultPolicyVersion', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return 1;
+   }
+}
+
+=head2 list_policy_versions(%params)
+
+Lists information about the versions of the specified managed policy, including the 
+version that is set as the policy's default version.
+
+=over
+
+=item PolicyArn (required)
+
+The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
+
+=item MaxItems (optional)
+
+Use this parameter only when paginating results to indicate the maximum number 
+of policy versions you want in the response.
+
+=item Marker (optional)
+
+Use this parameter only when paginating results, and only in a subsequent request 
+after you've received a response where the results are truncated. Set it to the value 
+of the Marker element in the response you just received.
+
+=back
+
+Returns L<Net::Amazon::IAM::PolicyVersions> on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub list_policy_versions {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      PolicyArn => { type => SCALAR },
+      MaxItems  => { type => SCALAR, optional => 1 },
+      Marker    => { type => SCALAR, optional => 1 },
+   });
+
+   my $xml = $self->_sign(Action => 'ListPolicyVersions', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      my %result   = %{$xml->{'ListPolicyVersionsResult'}};
+      my $versions = $self->_parse_attributes('PolicyVersion', 'Versions', %result);
+
+      return Net::Amazon::IAM::PolicyVersions->new(
+         Policies => $versions,
+      );
+   }
+}
+
+=head2 create_policy_version(%params)
+
+Creates a new version of the specified managed policy. To update a managed policy, 
+you create a new policy version. A managed policy can have up to five versions. 
+If the policy has five versions, you must delete an existing version using DeletePolicyVersion 
+before you create a new version.
+
+Optionally, you can set the new version as the policy's default version. The default version 
+is the operative version; that is, the version that is in effect for the IAM users, groups, 
+and roles that the policy is attached to.
+
+=over
+
+=item PolicyArn (required)
+
+The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
+
+=item PolicyDocument (required)
+
+The policy document.
+
+=item SetAsDefault (optional)
+
+Specifies whether to set this version as the policy's default version.
+
+When this parameter is true, the new policy version becomes the operative 
+version; that is, the version that is in effect for the IAM users, groups, 
+and roles that the policy is attached to.
+
+=back
+
+Returns L<Net::Amazon::IAM::PolicyVersion> on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub create_policy_version {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      PolicyArn      => { type => SCALAR },
+      PolicyDocument => { type => HASHREF },
+      SetAsDefault   => { regex => qr/true|false/s, optional => 1 },
+   });
+
+   $args{'PolicyDocument'} = encode_json delete $args{'PolicyDocument'};
+
+   my $xml = $self->_sign(Action => 'CreatePolicyVersion', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return Net::Amazon::IAM::PolicyVersion->new(
+         $xml->{'CreatePolicyVersionResult'}{'PolicyVersion'},
+      );
+   }
+}
+
+=head2 delete_policy_version(%params)
+
+=over
+
+=item PolicyArn (required)
+
+The Amazon Resource Name (ARN). ARNs are unique identifiers for AWS resources.
+
+=item VersionId (required)
+
+The policy version to delete.
+
+=back
+
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub delete_policy_version {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      PolicyArn => { type => SCALAR },
+      VersionId => { type => SCALAR },
+   });
+
+   my $xml = $self->_sign(Action => 'DeletePolicyVersion', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return 1;
+   }
+}
+
 =head2 put_user_policy(%params)
 
-Deletes the specified managed policy.
+Adds (or updates) an inline policy document that is embedded in the specified user.
 
 =over
 
@@ -1082,7 +1301,7 @@ The name of the user to associate the policy with.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1122,7 +1341,7 @@ The name of the user who the policy is associated with.
 
 =back
 
-Returns Net::Amazon::IAM::UserPolicy object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::UserPolicy> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1163,7 +1382,7 @@ The name (friendly name, not ARN) identifying the user that the policy is embedd
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1197,8 +1416,8 @@ The name of the user to list policies for.
 =back
 
 When found one or more policies, this method will return ArrayRef with policy names.
-Once no policies found, will return undef;
-Net::Amazon::IAM::Error will be returned on error
+Once no policies found, will return undef.
+L<Net::Amazon::IAM::Error> will be returned on error.
 
 =cut
 
@@ -1257,7 +1476,7 @@ The user name that the new key will belong to.
 
 =back
 
-Returns Net::Amazon::IAM::AccessKey object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::AccessKey> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1300,7 +1519,7 @@ The name of the user whose key you want to delete.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1349,7 +1568,7 @@ The name of the user whose key you want to update.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1387,9 +1606,9 @@ The name of the user.
 =back
 
 Returns Net::Amazon::IAM::AccessKeysList on success.
-If specified user has no keys, "Keys" attribute of Net::Amazon::IAM::AccessKeysList object
+If specified user has no keys, "Keys" attribute of L<Net::Amazon::IAM::AccessKeysList> object
 will be just empty array.
-Returns Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1446,7 +1665,7 @@ The path to the role.
 
 =back
 
-Returns Net::Amazon::IAM::Role object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Role> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1484,7 +1703,7 @@ The name of the role to get information about.
 
 =back
 
-Returns Net::Amazon::IAM::Role object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Role> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1540,7 +1759,7 @@ This parameter is optional. If it is not included, it defaults to a slash (/), l
 
 =back
 
-Returns Net::Amazon::IAM::Roles object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::Roles> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1587,7 +1806,7 @@ The name of the role to delete.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1607,6 +1826,50 @@ sub delete_role {
    }
 }
 
+=head2 put_role_policy(%params)
+
+Adds (or updates) an inline policy document that is embedded in the specified role.
+
+=over
+
+=item PolicyDocument (required)
+
+The policy document.
+
+=item PolicyName (required)
+
+The name of the policy document.
+
+=item RoleName (required)
+
+The name of the role to associate the policy with.
+
+=back
+
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub put_role_policy {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      PolicyDocument => { type => HASHREF },
+      PolicyName     => { type => SCALAR },
+      RoleName       => { type => SCALAR },
+   });
+
+   $args{'PolicyDocument'} = encode_json delete $args{'PolicyDocument'};
+
+   my $xml = $self->_sign(Action => 'PutRolePolicy', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return 1;
+   }
+}
+
 =head2 create_virtual_MFA_device(%params)
 
 Creates a new virtual MFA device for the AWS account. 
@@ -1633,7 +1896,7 @@ The path for the virtual MFA device.
 
 =back
 
-Returns Net::Amazon::IAM::VirtualMFADevice object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::VirtualMFADevice> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 B<This method wasn't tested>
 
@@ -1675,7 +1938,7 @@ For virtual MFA devices, the serial number is the same as the ARN.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 B<This method wasn't tested>
 
@@ -1726,7 +1989,7 @@ Valid Values: Assigned | Unassigned | Any
 
 =back
 
-Returns Net::Amazon::IAM::MFADevices object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::MFADevices> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1801,7 +2064,7 @@ The name of the user for whom you want to enable the MFA device.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 B<This method wasn't tested>
 
@@ -1845,7 +2108,7 @@ The name of the user whose MFA device you want to deactivate.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 B<This method wasn't tested>
 
@@ -1893,7 +2156,7 @@ The name of the user whose MFA devices you want to list.
 
 =back
 
-Returns Net::Amazon::IAM::MFADevices object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::MFADevices> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1957,8 +2220,7 @@ The path to the instance profile.
 
 =back
 
-Returns Net::Amazon::IAM::InstanceProfile object on success or Net::Amazon::IAM::Error on fail.
-The Roles attribute of Net::Amazon::IAM::InstanceProfile object will be undef.
+Returns L<Net::Amazon::IAM::InstanceProfile> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -1995,7 +2257,7 @@ The name of the instance profile to get information about.
 
 =back
 
-Returns Net::Amazon::IAM::InstanceProfile object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::InstanceProfile> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2056,7 +2318,7 @@ starts with /application_abc/component_xyz/.
 
 =back
 
-Returns Net::Amazon::IAM::InstanceProfiles object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::InstanceProfiles> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2109,7 +2371,7 @@ The name of the instance profile to delete.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2145,7 +2407,7 @@ The name of the role to add.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2189,7 +2451,7 @@ The name of the role to remove.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2235,7 +2497,7 @@ value of the Marker element in the response you just received.
 
 =back
 
-Returns Net::Amazon::IAM::InstanceProfiles object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::InstanceProfiles> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2296,7 +2558,7 @@ Specifies whether the user is required to set a new password on next sign-in.
 
 =back
 
-Returns Net::Amazon::IAM::LoginProfile object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::LoginProfile> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2339,7 +2601,7 @@ The name of the user whose password you want to delete.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2372,7 +2634,7 @@ The name of the user whose login profile you want to retrieve.
 
 =back
 
-Returns Net::Amazon::IAM::LoginProfile object on success or Net::Amazon::IAM::Error on fail.
+Returns L<Net::Amazon::IAM::LoginProfile> object on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2414,7 +2676,7 @@ Require the specified user to set a new password on next sign-in.
 
 =back
 
-Returns true on success or Net::Amazon::IAM::Error on fail.
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
 
 =cut
 
@@ -2436,6 +2698,253 @@ sub update_login_profile {
    }
 }
 
+=head2 update_assume_role_policy(%params)
+
+Updates the policy that grants an entity permission to assume a role.
+
+=over
+
+=item RoleName (required)
+
+The name of the role to update.
+
+=item PolicyDocument (required)
+
+The policy that grants an entity permission to assume the role.
+
+=back
+
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub update_assume_role_policy {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      RoleName       => { type => SCALAR },
+      PolicyDocument => { type => HASHREF },
+   }); 
+
+   $args{'PolicyDocument'} = encode_json delete $args{'PolicyDocument'};
+   
+   my $xml = $self->_sign(Action => 'UpdateAssumeRolePolicy', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return 1;
+   }
+}
+
+=head2 add_client_ID_to_open_ID_connect_provider(%params)
+
+Adds a new client ID (also known as audience) to the list of client IDs already registered for 
+the specified IAM OpenID Connect provider.
+
+This action is idempotent; it does not fail or return an error if you add an existing client 
+ID to the provider.
+
+=over
+
+=item ClientID (required)
+
+The client ID (also known as audience) to add to the IAM OpenID Connect provider.
+
+=item OpenIDConnectProviderArn (required)
+
+The Amazon Resource Name (ARN) of the IAM OpenID Connect (OIDC) provider to add the client ID to.
+
+=back
+
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
+
+B<This method wasn't tested>
+
+=cut
+
+sub add_client_ID_to_open_ID_connect_provider {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      ClientID                 => { type => SCALAR },
+      OpenIDConnectProviderArn => { type => SCALAR },
+   }); 
+
+   my $xml = $self->_sign(Action => 'AddClientIDToOpenIDConnectProvider', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return 1;
+   }
+}
+
+=head2 create_open_ID_connect_provider(%params)
+
+Creates an IAM entity to describe an identity provider (IdP) that supports OpenID Connect (OIDC).
+
+=over
+
+=item ClientIDList (required)
+
+A list of client IDs (also known as audiences). When a mobile or web app registers with 
+an OpenID Connect provider, they establish a value that identifies the application. 
+(This is the value that's sent as the client_id parameter on OAuth requests.)
+
+You can register multiple client IDs with the same provider. For example, you might have 
+multiple applications that use the same OIDC provider. You cannot register more than 100 
+client IDs with a single IAM OIDC provider.
+
+=item ThumbprintList (required)
+
+A list of server certificate thumbprints for the OpenID Connect (OIDC) identity provider's 
+server certificate(s). Typically this list includes only one entry. However, IAM lets you 
+have up to five thumbprints for an OIDC provider. This lets you maintain multiple thumbprints 
+if the identity provider is rotating certificates.
+
+The server certificate thumbprint is the hex-encoded SHA-1 hash value of the X.509 certificate 
+used by the domain where the OpenID Connect provider makes its keys available. It is always a 
+40-character string.
+
+You must provide at least one thumbprint when creating an IAM OIDC provider. For example, if the 
+OIDC provider is server.example.com and the provider stores its keys at 
+"https://keys.server.example.com/openid-connect", the thumbprint string would be the hex-encoded 
+SHA-1 hash value of the certificate used by https://keys.server.example.com.
+
+=item Url (required)
+
+The URL of the identity provider. The URL must begin with "https://" and should correspond to 
+the iss claim in the provider's OpenID Connect ID tokens. Per the OIDC standard, path components 
+are allowed but query parameters are not. Typically the URL consists of only a host name, like 
+"https://server.example.org" or "https://example.com".
+
+You cannot register the same provider multiple times in a single AWS account. If you try to 
+submit a URL that has already been used for an OpenID Connect provider in the AWS account, 
+you will get an error.
+
+=back
+
+Returns OpenIDConnectProviderArn on success or L<Net::Amazon::IAM::Error> on fail.
+
+B<This method wasn't tested>
+
+=cut
+
+sub create_open_ID_connect_provider {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      ClientIDList   => { type => ARRAYREF, optional => 1 },
+      ThumbprintList => { type => ARRAYREF },
+      Url            => { type => SCALAR },
+   }); 
+
+   my $client_ids_list  = delete $args{'ClientIDList'};
+   my $thumb_print_list = delete $args{'ThumbprintList'};
+
+   my $c_count = 1;
+   for my $id(@{$client_ids_list}) {
+      $args{'ClientIDList.list.' . $c_count} = $id;
+      $c_count++;
+   }
+
+   my $t_count = 1;
+   for my $thumb(@{$thumb_print_list}) {
+      $args{'ThumbprintList.list.' . $t_count} = $thumb;
+      $t_count++;
+   }
+
+   my $xml = $self->_sign(Action => 'CreateOpenIDConnectProvider', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return $xml->{'CreateOpenIDConnectProviderResult'}{'OpenIDConnectProviderArn'}
+   }
+}
+
+=head2 delete_open_ID_connect_provider(%params)
+
+Deletes an IAM OpenID Connect identity provider.
+
+Deleting an OIDC provider does not update any roles that reference the provider as a 
+principal in their trust policies. Any attempt to assume a role that references a 
+provider that has been deleted will fail.
+
+This action is idempotent; it does not fail or return an error if you call the action 
+for a provider that was already deleted.
+
+=over
+
+=item OpenIDConnectProviderArn (required)
+
+The Amazon Resource Name (ARN) of the IAM OpenID Connect provider to delete.
+
+=back
+
+Returns true on success or L<Net::Amazon::IAM::Error> on fail.
+
+B<This method wasn't tested>
+
+=cut
+
+sub delete_open_ID_connect_provider {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      OpenIDConnectProviderArn   => { type => SCALAR },
+   }); 
+
+   my $xml = $self->_sign(Action => 'DeleteOpenIDConnectProvider', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      return 1;
+   }
+}
+
+=head2 get_group_policy(%params)
+
+=over
+
+=item GroupName (required)
+
+The name of the group the policy is associated with.
+
+=item PolicyName (required)
+
+The name of the policy document to get.
+
+=back
+
+Returns L<Net::Amazon::IAM::GroupPolicy> on success or L<Net::Amazon::IAM::Error> on fail.
+
+=cut
+
+sub get_group_policy {
+   my $self = shift;
+
+   my %args = validate(@_, {
+      GroupName  => { type => SCALAR },
+      PolicyName => { type => SCALAR },
+   }); 
+
+   my $xml = $self->_sign(Action => 'GetGroupPolicy', %args);
+
+   if ( grep { defined && length } $xml->{'Error'} ) {
+      return $self->_parse_errors($xml);
+   } else {
+      my %result = ${$xml->{'GetGroupPolicyResult'}};
+      $result{'PolicyDocument'} = decode_json(URI::Encode->new()->decode($result{'PolicyDocument'}));
+
+      return Net::Amazon::IAM::GroupPolicy->new(
+         %result,
+      );
+   }
+}
+
 no Moose;
 1;
 
@@ -2447,6 +2956,16 @@ no Moose;
 
 * list_user_policies returns just an ArrayRef.
 
+=head1 SEE ALSO
+
+=over
+
+=item Amazon IAM API reference
+
+http://docs.aws.amazon.com/IAM/latest/APIReference/Welcome.html
+
+=back
+
 =head1 AUTHOR
 
 Igor Tsigankov <tsiganenok@gmail.com>
