@@ -1,6 +1,42 @@
 package Net::Amazon::IAM::Group;
 use Moose;
 
+=head1 NAME
+
+Net::Amazon::IAM::Group
+
+=head1 DESCRIPTION
+
+A class representing a IAM Group.
+
+=head1 ATTRIBUTES
+
+=over
+
+=item Arn (required)
+
+The Amazon Resource Name (ARN) specifying the group.
+
+=item CreateDate (required)
+
+The date and time, in ISO 8601 date-time format, when the group was created.
+
+=item GroupId (required)
+
+The stable and unique string identifying the group.
+
+=item GroupName (required)
+
+The friendly name that identifies the group.
+
+=item Path (required)
+
+The path to the group. 
+
+=back
+
+=cut
+
 has 'Arn' => (
    is       => 'ro',
    isa      => 'Str',
@@ -31,18 +67,6 @@ has 'Path' => (
    required => 1,
 );
 
-has 'IsTruncated' => (
-   is       => 'ro',
-   isa      => 'Str',
-   required => 0,
-);
-
-has 'Users' => (
-   is       => 'ro',
-   isa      => 'Maybe[ArrayRef[Net::Amazon::IAM::User]]',
-   required => 0,
-);
-
 __PACKAGE__->meta->make_immutable();
 
 =head1 AUTHOR
